package ACL2::Proceso::Contradiccion;
use base ACL2::Proceso;

=head1 NAME

ACL2::Proceso::Contradiccion - ACL2 sospecha que la fE<oacute>rmula es una contradicciE<oacute>n

=head1 FUNCTIONS

=cut

#
#    Clase Contradiccion: ACL2 sospecha que la fórmula es una contradicción
#
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Orden;
use ACL2::Proceso::Fabrica;

BEGIN {
ACL2::Proceso::Fabrica->registrar
    (
     __PACKAGE__,
     sub {
	 return ACL2::Proceso::Contradiccion->_identificar(@_);
     }
     );
}

sub escribir_xml_cabecera {
    my ($this, $writer) = @_;

    $writer->startTag(ACL2::Tags::NOT_A_THEOREM_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{salida});
}

sub escribir_xml_final {
    my ($this, $writer) = @_;
    $writer->endTag(ACL2::Tags::NOT_A_THEOREM_TAG);
}

sub _identificar
{
    my ($paquete, $salida) = @_;
    my %res      = ();
    
    $salida =~ /^\s*(?-x:We suspect that this conjecture is not a theorem\.)/ox
	or return undef;
    
    $res{salida} = substr $salida, $-[0], $+[0] - $-[0];
    return \%res;
}

1;
