package ACL2::Proceso::Induccion;
use base ACL2::Proceso;

=head1 NAME

ACL2::Proceso::Induccion - ACL2 realiza una inducciE<oacute>n para demostrar una fE<oacute>rmula

=head1 FUNCTIONS

=cut


#    Clase Induccion: ACL2 realiza una inducción para demostrar una
#    fórmula.  En esta clase no nos ocupamos de las submetas: de eso
#    se ocupan ACL2::Resultado y ACL2::Meta.
#
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Orden;
use ACL2::Localizacion;
use ACL2::Proceso::Fabrica;

our $lh = ACL2::Localizacion->get_handle;

BEGIN {
    ACL2::Proceso::Fabrica->registrar
    (
     __PACKAGE__,
     sub {
	 return ACL2::Proceso::Induccion->_identificar(@_);
     }
     );
}

sub escribir_xml_cabecera {
    my ($this, $writer) = @_;

    if ($this->{fracaso}) {
	# Fracaso: ACL2 no halló un plan de inducción útil
	$writer->startTag(ACL2::Tags::FAILURE_TAG);
	ACL2::Orden->salida_parrafos($writer, $this->{salida});
	return;
    }

    # Inicio
    $writer->startTag(ACL2::Tags::INDUCT_TAG,
		      "plan" => $this->{plan});

    # Adaptación
    if (exists $this->{adaptacion}) {
	$writer->startTag(ACL2::Tags::INDSUGGS_TAG, 
			  "hint" => $this->{pista_ind}, 
			  "adapt" => $this->{adaptacion});
    } else {
	$writer->startTag(ACL2::Tags::INDSUGGS_TAG, 
			  "hint" => $this->{pista_ind});
    }

    # Sugerencias
    foreach (@{$this->{sugerencias}}) {
	$writer->emptyTag(ACL2::Tags::RULE_TAG, 
			  "type" => $_->{type}, 
			  "name" => $_->{name});
    }
    $writer->endTag(ACL2::Tags::INDSUGGS_TAG); 

    # Salida
    ACL2::Orden->salida_parrafos($writer, $this->{salida});
}

sub escribir_xml_final {
    my ($this, $writer) = @_;

    $writer->endTag($this->{fracaso} 
		    ? ACL2::Tags::FAILURE_TAG
		    : ACL2::Tags::INDUCT_TAG);
}

sub _identificar
{
    my ($paquete, $salida) = @_;
    my %res      = ();

    $res{salida} = $salida;

    ## Sugerencia de inducción
    for ($salida) {
	/(?-x:We will induct according to a scheme suggested by (\(.*?\)))
	 (?-x:, but modified to accommodate (.*?)\.)/ogsx && do {
	     $res{pista_ind}  = $1;
	     $res{adaptacion} = $2;
	     last;
	};
	
	/We will induct according to a scheme suggested by (\(.*?\))/ogs && do {
	    $res{pista_ind} = $1;
	    last;
	};
	
	/No induction schemes are suggested by .*?\.\s/ && do {
	    $res{fracaso} = 1;
	    $res{salida}  = $salida;
	    return \%res;
	};

	# No se sigue un esquema conocido de inducción
	return undef;
    }

    # Obtención de las sugerencias
    $salida =~ /(?-x:These suggestions were|This suggestion was)
	(?-x: produced using ([^\.]*)\. If we let (\(.*?\)) denote .*? above)
	/oxs
	or die $lh->maketext("Se esperaba encontrar inducción.");
    $res{sugerencias} = ACL2::Proceso->obtener_reglas($1);
    
    # Plan de inducción
    $salida =~ /then the induction scheme we\'ll use is\s+([^\.]+)\./gs
	or die $lh->maketext("Se esperaba encontrar el plan de inducción.");
    $res{plan} = $1;

    return \%res;
}

1;
